# Working_Memory_test.sce
#
# Working-Memory-Test, Implementation auf Basis der Matlab-Version
# der Working-Memory-Test-Batterie von Stephan Lewandowsky, Klaus Oberauer,
# Lee-Xieng Yang, and Ullrich Ecker portiert für Presentation.
#
# nur Operation-Span und Sentence-Span implementiert.
#
# Humboldt-Universität zu Berlin
# Institut für deutsche Sprache und Linguistik
# 28.06.2011 @ Dipl.-Ing. Carsten Schliewe 
#
#
# 26 Tasten über PC-Tastatur (Tastatur mit hoher Polling-Rate empfohlen)
# Tasten A bis Z für Eingabe Buchstaben
# Taste Z = yeskey
# Taste G = nokey


# Header
no_logfile = false;							# automatisches Logfile erstellen							
response_matching = simple_matching;	# Responses auswerten 
default_font_size = 26;                # Default-Text-Größe
default_font = "Arial";						# Default-Text-Font
default_text_align = align_left;       # Default-Ausrichtung Text
default_background_color = 255,255,255;# Default Hintergrundfarbe (weiss)
default_text_color = 0, 0, 0;          # Default-Textfarbe (schwarz)
active_buttons = 27;							# Auswertung von 26 Tasten
response_port_output = false;			   # Wenn dieser Parameter auf "false" steht gibt es keine Port-Auasgaben durch "button-Codes" mehr
response_logging = log_active;			# verhindert, dass deaktivierte Tasten mitgeloggt werden

### Beginn des Programmes
begin;

# Variablen definieren
$green  = "  0,255,0";      # Farbe grün
$red    = "255,  0,0";		 # Farbe rot
$gray   = "128,128,128";	 # Farbe grau
$white = "255,255,255";		 # Farbe weiss
$black = "0,0,0";		 		 # Farbe Schwarz

picture {} default;			 # Default-Bild (Nichts)

# Fixkreuz besteht aus 2 Box-Pictureparts
picture {box {height = 30; width = 5; color = $black;}; x = 0; y = 0;
         box {height = 5; width = 30; color = $black;}; x = 0; y = 0;} fix; # Frei wählbarer Name des Bildes
         
         
# Im pcl-Teil des Programms erfolgt hier später die von Text oder Gleichung
text {caption = "?"; font_size = 36; preload = false;} txt1;

picture {text txt1; x = 0; y = 0;} pic1;


# Trial für Blank Fix Blank  
trial {trial_duration = stimuli_length;
       picture default;				 
       time = 0;
       duration = 500;
       code = "blank_v_fix";
       picture fix;
       time = 500;
       duration = 500;
		 code = "fix";
       picture default;				 
       time = 1000;
       duration = 100;
       code = "blank_n_fix";
       } trial_Bl_Fix_Bl;
       
# Instruktion-Trial für Einführung und Instruktionen für Text
trial {trial_type = first_response;
       trial_duration = forever;
       picture {text {caption = "?"; font_size = 30; text_align = align_center;} instr_txt; x = 0; y = 100;
                text {caption = "Weiter mit Leertaste";}; x = 0; y = -400;}pic_Instr;
                } Instruktion; 

# Instruktion-Trial für Einführung und Instruktionen für Bilder
trial {trial_type = fixed;																	
       trial_duration = stimuli_length;																		
       picture {        bitmap {
	        system_memory = true;
	        scale_factor = 1.4;
	        filename = "InitInstruct.jpg";
        }pic_instr;
                x = 0; y = 0;};
} Instruktion_1; 					# Name des Trials

                
# Trial für Gleichungen und Sätze
trial {	trial_type = first_response;
			trial_duration = 3000;
			all_responses = false;		# Reaktionen erst beim "pic1" zulassen
			stimulus_event{
			picture pic1;					
			time = 0;
			code = "pic";
			response_active = true;    # Erst jetzt Reaktionen durch den Probanden erlauben, ab jetzt läuft die Uhr. 
			} event_pic;	
		} trial_Gleichung_Satz;

# Trial für zu merkende Buchstaben
trial {	trial_type = fixed;
			trial_duration = 1000;
			all_responses = false;		
			stimulus_event{
			picture pic1;					
			time = 0;
			code = "Buchst";
			response_active = true;    # Erst jetzt Reaktionen durch den Probanden erlauben, ab jetzt läuft die Uhr. 
			} event_Buchst;	
		} trial_Buchstabe;		

# Trial für Fragezeichen (Abfrage der Buchstaben am Ende)
trial {	trial_type = first_response;
			trial_duration = forever;
			all_responses = false;		# Reaktionen erst beim "pic1" zulassen
			stimulus_event{
			picture pic1;					
			time = 0;
			code = "Fragez";
			response_active = true;    # Erst jetzt Reaktionen durch den Probanden erlauben, ab jetzt läuft die Uhr. 
			} event_Fragez;	
		} trial_Fragez;
		
# Trial für zu merkende Buchstaben
trial {	trial_type = fixed;
			trial_duration = 200;
			picture pic1;					
			time = 0;
			code = "Buchst_resp";
		} trial_Buchst_resp;

# Ab hier beginnt die PCL-Programmiersprache
begin_pcl;

# In dieser Datei sind u.a. unsere Gleichungen und Sätze definiert
include "Items1_Test.pcl";


# VP-Nummer in Eingabemaske eingeben
preset int VP_Name; 

# Logdatei für Operation-Spam-Test öffnen (Datensätze anhängen)
output_file file = new output_file;
file.open_append("OS.dat");

# Logdatei für Lesespannen-Test öffnen (Datensätze anhängen)
output_file file1 = new output_file;
file1.open_append("SS.dat");


# Subroutine, Aufgabe: Schaltet nur die Tasten Z ung G (yeskey und nokey) frei
sub Only_Z_and_G
begin
   response_manager.set_button_active(1,true);
   response_manager.set_button_active(2,true);
	loop
		int i = 3
	until i > 27
	begin
			response_manager.set_button_active(i,false);
			i = i +1;
  end;	
end;

# Subroutine, Aufgabe: Schaltet alle Buchstaben-Tasten frei
sub All_Letters
begin
   # Leertaste deaktivieren
   response_manager.set_button_active(27,false);#
	loop
		int i = 1
	until i > 26
	begin
			response_manager.set_button_active(i,true);
			i = i +1;
  end;	
end;

# Subroutine, Aufgabe: Schaltet nur die Leertaste frei
sub Only_Space
begin
	response_manager.set_button_active(27,true);
	loop
		int i = 1
	until i > 26
	begin
			response_manager.set_button_active(i,false);
			i = i +1;
  end;	
end;

# Subroutine, Aufgabe: Ordnet dem  Resonse-key (integer-Wert) den unter "Settings, Response" zugeordneten Buchstaben der PC-Tastatur zu
# gibt den Wert dann als String zurück 
sub
string keyToLetter(int key)
begin
if (key == 0 || key > 26) then exit("Diesem Response key ist kein Buchstabe zugeordnet, bitte unter Settings Response kontrollieren") end;
array <string> LetterArray[26] = {"G","Z","Q","W","E","R","T","U","I","O","P","A","S","D","F","H","J","K","L","Y","X","C","V","B","N","M"};
return LetterArray[key];
end;


# Subroutine für das Rechen-Experiment (Darbietung der Trials). Nur für Übung
# Wird aus "Hauptprogramm" (siehe unten) aufgerufen und erst dann ausgeführt.          
sub do_OS_ueb
begin
# loop-Schleife wird solange ausgeführt bis Bedingung unter "until" erfüllt ist
loop
	int i = 1; # Laufindex zum Zugriff auf die Zeilen des Arrays
until i > OS_ueb.count() # Loop (und sub) verlassen wenn Laufvariable grösser als angegebene Zeilenanzahl im Array ist. 
begin
# Nur yes_key und no_key freigeben
Only_Z_and_G();
trial_Bl_Fix_Bl.present();
		loop
			int j = 1;
		until j > int(OS_ueb [i][25])
		begin
			event_pic.set_event_code("OS_ueb"+string(i)+"_"+string(j));
			txt1.set_caption(OS_ueb [i][8+j]);
			txt1.redraw();
			trial_Gleichung_Satz.set_duration(3000);
			trial_Gleichung_Satz.present();
			default.present();
			wait_interval(100);
			txt1.set_caption(OS_ueb [i][j]);
			txt1.redraw();
			trial_Buchstabe.present();
			j = j + 1;
		end;
		# Jetzt wieder alle Buchstaben freigeben
		All_Letters();
		loop
			int j = 1;
		until j > int(OS_ueb [i][25])
		begin
		   int key1 = 0;
			event_Fragez.set_event_code("?_OS_ueb"+string(i)+"_"+string(j));
			txt1.set_caption("?");
			txt1.redraw();
			int responses = response_manager.total_response_count();
			trial_Fragez.present();
			if response_manager.total_response_count() > responses then
				key1 = response_manager.last_response();
			end;
			txt1.set_caption(keyToLetter(key1));
			#term.print_line(system_keyboard.get_keypress_time(system_keyboard.keypress_count( )));
			txt1.redraw();
			trial_Buchst_resp.present();
		j = j + 1;
		end;
i=i+1;
end;
end;  # von do_OS_ueb

# Subroutine für das Rechen-Experiment (Darbietung der Trials)
# Wird aus "Hauptprogramm" (siehe unten) aufgerufen und erst dann ausgeführt.          
sub do_OS_exp
begin
# loop-Schleife wird solange ausgeführt bis Bedingung unter "until" erfüllt ist
loop
	int i = 1; # Laufindex zum Zugriff auf die Zeilen des Arrays
until i > OS.count() # Loop (und sub) verlassen wenn Laufvariable grösser als angegebene Zeilenanzahl im Array ist. 
begin
	 # Dieses Array nimmt die Reaktionszeiten für die Bewertung der Gleichungen / Sätze auf
	 array <string> RT_Ri_Wr[8] = {"-1.000","-1.000","-1.000","-1.000","-1.000","-1.000","-1.000","-1.000"};
	 # Dieses Array nimmt die Reaktionszeiten für die Eingabe der Buchstaben auf
	 array <string> RT_Letter[8] = {"-1.000","-1.000","-1.000","-1.000","-1.000","-1.000","-1.000","-1.000"};
	 array <string> yes_no_key[8] = {"-1","-1","-1","-1","-1","-1","-1","-1"};
	 array <string> Ans_Letter[8] = {"%","%","%","%","%","%","%","%"};
    # Nur Leertaste freigeben
	 Only_Space();
    if ((i == 4) || (i == 7) || (i == 10) || (i == 13)) then 
		instr_txt.set_caption("Drücken Sie die Leertaste nach einer Pause !");
		instr_txt.redraw();
		Instruktion.present();  
    end;
    trial_Bl_Fix_Bl.present();
		loop
			int j = 1;
		until j > int(OS[i][25])
		begin
		   # Nur yes_key und no_key freigeben
			Only_Z_and_G();
			int key = 0;
			event_pic.set_event_code("OS"+string(i)+"_"+string(j));
			txt1.set_caption(OS[i][8+j]);
			txt1.redraw();
			# Responses vor Trial-Ausführung merken
			int responses = response_manager.total_response_count();
			trial_Gleichung_Satz.set_duration(3000);
			trial_Gleichung_Satz.present();
			if response_manager.total_response_count() > responses then
				#term.print_line(stimulus_manager.last_stimulus_data().reaction_time_double());
				RT_Ri_Wr[j] = string(round((stimulus_manager.last_stimulus_data().reaction_time_double()/1000.0),3));
				key = response_manager.last_response();
            if key == 1 then
             yes_no_key[j]="0"
				elseif key == 2 then
				 yes_no_key[j]="1"
				else
				 yes_no_key[j]="-9"
				end;
			else
				RT_Ri_Wr[j] ="0.000";
				yes_no_key[j]="-9"
			end;
			default.present();
			wait_interval(100);
			txt1.set_caption(OS[i][j]);
			txt1.redraw();
			trial_Buchstabe.present();
			j = j + 1;
		end;
		All_Letters();
		loop
			int j = 1;
		until j > int(OS[i][25])
		begin
		   double RT_Buchst_d = 0.0;
		   int key1 = 0;
			event_Fragez.set_event_code("?_OS"+string(i)+"_"+string(j));
			txt1.set_caption("?");
			txt1.redraw();
			int responses = response_manager.total_response_count();
			trial_Fragez.present();
			if response_manager.total_response_count() > responses then
				RT_Buchst_d=stimulus_manager.last_stimulus_data().reaction_time_double();
				key1 = response_manager.last_response();
			end;
			#term.print_line(key1);
			Ans_Letter[j] = keyToLetter(key1);
			#term.print_line(Ans_Letter[j]);
			txt1.set_caption(Ans_Letter[j]);
         RT_Letter[j]=string(round((RT_Buchst_d/1000.0),3));  
			txt1.redraw();
			trial_Buchst_resp.present();
		j = j + 1;
		end;
		# Für jedes Item eine Zeile in das Logfile schreiben
		# Logilename
		file.print(string(VP_Name));
		file.print(" ");
		# Itemnummer
		file.print(string(i));
		file.print(" ");
		# Anzahl der Gleichungen / Buchstaben
		file.print(OS[i][25]);
		file.print(" ");
		# Anzuzeigende Buchstaben ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file.print(OS[i][j]+" ");
		j=j+1
		end;
		file.print("     ");
		# Eingegebene Buchstaben ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file.print(Ans_Letter[j]+" ");
		j=j+1
		end;
		file.print("     ");
		# Reaktionszeiten (Eingabe Buchstaben) ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file.print(RT_Letter[j]+" ");
		j=j+1
		end;
		file.print("     ");
		# Vorgabe Gleichung ist Richtig oder falsch ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file.print(OS[i][16+j]+"  ");
		j=j+1
		end;
		file.print("     ");		
      # Bewertung Proband Gleichung ist Richtig oder falsch ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file.print(yes_no_key[j]+"  ");
		j=j+1
		end;
		file.print("     ");		
		# Reaktionszeit der Richtig/Falsch-Bewertung ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file.print(RT_Ri_Wr[j]+" ");
		j=j+1
		end;
		# Nächste Zeile
		file.print("\n");
i=i+1;
end;
end;  # von do_OS_exp


# Subroutine für das Sätze-Experiment (Darbietung der Trials). Nur für Übung
# Wird aus "Hauptprogramm" (siehe unten) aufgerufen und erst dann ausgeführt.          
sub do_SS_ueb
begin
# loop-Schleife wird solange ausgeführt bis Bedingung unter "until" erfüllt ist
loop
	int i = 1; # Laufindex zum Zugriff auf die Zeilen des Arrays
until i > SS_ueb.count() # Loop (und sub) verlassen wenn Laufvariable grösser als angegebene Zeilenanzahl im Array ist. 
begin
# Nur yes_key und no_key freigeben
Only_Z_and_G();
trial_Bl_Fix_Bl.present();
		loop
			int j = 1;
		until j > int(SS_ueb [i][25])
		begin
			event_pic.set_event_code("SS_ueb"+string(i)+"_"+string(j));
			txt1.set_caption(SS_ueb [i][8+j]);
			txt1.redraw();
			trial_Gleichung_Satz.set_duration(4000);
			trial_Gleichung_Satz.present();
			default.present();
			wait_interval(100);
			txt1.set_caption(SS_ueb [i][j]);
			txt1.redraw();
			trial_Buchstabe.present();
			j = j + 1;
		end;
		# Jetzt wieder alle Buchstaben freigeben
		All_Letters();
		loop
			int j = 1;
		until j > int(SS_ueb [i][25])
		begin
		   int key1 = 0;
			event_Fragez.set_event_code("?_SS_ueb"+string(i)+"_"+string(j));
			txt1.set_caption("?");
			txt1.redraw();
			int responses = response_manager.total_response_count();
			trial_Fragez.present();
			if response_manager.total_response_count() > responses then
				key1 = response_manager.last_response();
			end;
			txt1.set_caption(keyToLetter(key1));
			#term.print_line(system_keyboard.get_keypress_time(system_keyboard.keypress_count( )));
			txt1.redraw();
			trial_Buchst_resp.present();
		j = j + 1;
		end;
i=i+1;
end;
end;  # von do_SS_ueb

# Subroutine für das Rechen-Experiment (Darbietung der Trials)
# Wird aus "Hauptprogramm" (siehe unten) aufgerufen und erst dann ausgeführt.          
sub do_SS_exp
begin
# loop-Schleife wird solange ausgeführt bis Bedingung unter "until" erfüllt ist
loop
	int i = 1; # Laufindex zum Zugriff auf die Zeilen des Arrays
until i > SS.count() # Loop (und sub) verlassen wenn Laufvariable grösser als angegebene Zeilenanzahl im Array ist. 
begin
	 # Dieses Array nimmt die Reaktionszeiten für die Bewertung der Gleichungen / Sätze auf
	 array <string> RT_Ri_Wr[8] = {"-1.000","-1.000","-1.000","-1.000","-1.000","-1.000","-1.000","-1.000"};
	 # Dieses Array nimmt die Reaktionszeiten für die Eingabe der Buchstaben auf
	 array <string> RT_Letter[8] = {"-1.000","-1.000","-1.000","-1.000","-1.000","-1.000","-1.000","-1.000"};
	 array <string> yes_no_key[8] = {"-1","-1","-1","-1","-1","-1","-1","-1"};
	 array <string> Ans_Letter[8] = {"%","%","%","%","%","%","%","%"};
    # Nur Leertaste freigeben
	 Only_Space();
    if ((i == 4) || (i == 7) || (i == 10) || (i == 13)) then 
		instr_txt.set_caption("Drücken Sie die Leertaste nach einer Pause !");
		instr_txt.redraw();
		Instruktion.present();  
    end;
    trial_Bl_Fix_Bl.present();
		loop
			int j = 1;
		until j > int(SS[i][25])
		begin
		   int key = 0;
		   # Nur yes_key und no_key freigeben
			Only_Z_and_G();
			event_pic.set_event_code("SS"+string(i)+"_"+string(j));
			txt1.set_caption(SS[i][8+j]);
			txt1.redraw();
			# Responses vor Trial-Ausführung merken
			int responses = response_manager.total_response_count();
			trial_Gleichung_Satz.set_duration(4000);
			trial_Gleichung_Satz.present();
			if response_manager.total_response_count() > responses then
				#term.print_line(stimulus_manager.last_stimulus_data().reaction_time_double());
				RT_Ri_Wr[j] = string(round((stimulus_manager.last_stimulus_data().reaction_time_double()/1000.0),3));
				key = response_manager.last_response();
            if key == 1 then
             yes_no_key[j]="0"
				elseif key == 2 then
				 yes_no_key[j]="1"
				else
				 yes_no_key[j]="-9"
				end;
			else
				RT_Ri_Wr[j] ="0.000";
				yes_no_key[j]="-9";
			end;
			default.present();
			wait_interval(100);
			txt1.set_caption(SS[i][j]);
			txt1.redraw();
			trial_Buchstabe.present();
			j = j + 1;
		end;
		All_Letters();
		loop
			int j = 1;
		until j > int(SS[i][25])
		begin
		   double RT_Buchst_d = 0.0;
		   int key1 = 0;
			event_Fragez.set_event_code("?_SS"+string(i)+"_"+string(j));
			txt1.set_caption("?");
			txt1.redraw();
			int responses = response_manager.total_response_count();
			trial_Fragez.present();
			if response_manager.total_response_count() > responses then
				RT_Buchst_d=stimulus_manager.last_stimulus_data().reaction_time_double();
				key1 = response_manager.last_response();
			end;
			#term.print_line(key1);
			Ans_Letter[j] = keyToLetter(key1);
			#term.print_line(Ans_Letter[j]);
			txt1.set_caption(Ans_Letter[j]);
         RT_Letter[j]=string(round((RT_Buchst_d/1000.0),3));  
			txt1.redraw();
			trial_Buchst_resp.present();
		j = j + 1;
		end;
		# Für jedes Item eine Zeile in das Logfile schreiben
		# Logilename
		file1.print(string(VP_Name));
		file1.print(" ");
		# Itemnummer
		file1.print(string(i));
		file1.print(" ");
		# Anzahl der Gleichungen / Buchstaben
		file1.print(SS[i][25]);
		file1.print(" ");
		# Anzuzeigende Buchstaben ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file1.print(SS[i][j]+" ");
		j=j+1
		end;
		file1.print("     ");
		# Eingegebene Buchstaben ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file1.print(Ans_Letter[j]+" ");
		j=j+1
		end;
		file1.print("     ");
		# Reaktionszeiten (Eingabe Buchstaben) ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file1.print(RT_Letter[j]+" ");
		j=j+1
		end;
		file1.print("     ");
		# Vorgabe Gleichung ist Richtig oder falsch ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file1.print(SS[i][16+j]+"  ");
		j=j+1
		end;
		file1.print("     ");		
      # Bewertung Proband Gleichung ist Richtig oder falsch ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file1.print(yes_no_key[j]+"  ");
		j=j+1
		end;
		file1.print("     ");		
		# Reaktionszeit der Richtig/Falsch-Bewertung ins Logfile schreiben
		loop
			int j = 1;
		until j > 8
		begin
		file1.print(RT_Ri_Wr[j]+" ");
		j=j+1
		end;
		# Nächste Zeile
		file1.print("\n");
i=i+1;
end;
end;  # von do_SS_exp


### Hauptprogramm

# Instruktion für Aufgabe darbieten (mit Trial)  
Instruktion_1.present();
system_keyboard.set_max_length( 1 );
string a = system_keyboard.get_input();
pic_instr.set_filename("OSInstruct1.jpg");
pic_instr.load();
Instruktion_1.present();
a = system_keyboard.get_input();
pic_instr.set_filename("OSInstruct2.jpg");
pic_instr.set_load_size(0.0,0.0,1.3);
pic_instr.load();
Instruktion_1.present();
a = system_keyboard.get_input();
pic_instr.set_filename("OSInstruct3.jpg");
pic_instr.load();
Instruktion_1.present();
a = system_keyboard.get_input();
Only_Space();
instr_txt.set_caption("Bitte Leertaste drücken um Übung zu starten");
instr_txt.redraw();
Instruktion.present();
do_OS_ueb();
Only_Space();
instr_txt.set_caption("Bitte Leertaste drücken um die Aufgabe zu starten");
instr_txt.redraw();
Instruktion.present();
do_OS_exp();
Only_Space();
instr_txt.set_caption("Aufgabe abgeschlossen!\n\n Drücken Sie die Leertaste, um mit der nächsten Aufgabe zu starten");
instr_txt.redraw();
Instruktion.present();
pic_instr.set_filename("SSeasyInstruct1.jpg");
pic_instr.load();
Instruktion_1.present();
a = system_keyboard.get_input();
pic_instr.set_filename("SSeasyInstruct2.jpg");
pic_instr.load();
Instruktion_1.present();
a = system_keyboard.get_input();
instr_txt.set_caption("Bitte Leertaste drücken um Übung zu starten");
instr_txt.redraw();
Instruktion.present();
do_SS_ueb();
Only_Space();
instr_txt.set_caption("Bitte Leertaste drücken um die Aufgabe zu starten");
instr_txt.redraw();
Instruktion.present();
do_SS_exp();
Only_Space();
instr_txt.set_caption("Experiment abgeschlossen!\n\nBitte Melden Sie sich beim Versuchsleiter");
instr_txt.redraw();
Instruktion.present();
